var obj = document.getElementById('drawing');
var ctx = obj.getContext('2d');
var Idint;
var points = [];
var colors = ["black", "green", "blue"];
var isRun = false;

addEventListener("mousedown", mDown);
addEventListener("mouseup", mUp);
addEventListener("mousemove", mMove);

var mouseDown = false;
var dragPoint = 0;
function mDown(event){
  let rect = obj.getBoundingClientRect();
  let x = event.clientX - rect.left;
  let y = event.clientY - rect.top;
  for(let i = 0; i < points.length; i++){
    if(x >= points[i].x - points[i].r && x <= points[i].x + points[i].r && y >= points[i].y - points[i].r && y <= points[i].y + points[i].r){
      points[i].x = x; points[i].y = y; dragPoint = i;
      mouseDown = true;
      bezierProc(0);
    }
  }
}
function mUp(event){
  mouseDown = false;
}

function mMove(event){
  if(mouseDown){
    let rect = obj.getBoundingClientRect();
    let x = event.clientX - rect.left;
    let y = event.clientY - rect.top;
    let ind = dragPoint;
    if(x < 5) x = 5;
    if(x > obj.width - 5) x = obj.width - 5;
    if(y < 5) y = 5;
    if(y > obj.height - 5) y = obj.height - 5;
    points[ind].x = x; points[ind].y = y;
    bezierProc(0);
  }
}

class Point{
  x = 0; y = 0; legend;
  r = 5; color;
  
  constructor(x = 0, y = 0, legend = "", color = "black"){
    this.x = x; this.y = y;
    this.legend = legend;
    this.color = color;
  }
  
  draw(){
    ctx.beginPath();
    ctx.fillStyle = "white";
    ctx.arc(this.x, this.y, this.r, 0, 2*Math.PI);
    ctx.fill();
    ctx.strokeStyle = this.color;
    ctx.arc(this.x, this.y, this.r, 0, 2*Math.PI);
    ctx.stroke();
    ctx.fillStyle = this.color;
    ctx.font = '14px sherif'
    ctx.fillText(this.legend, this.x - this.r - 2, this.y - this.r - 2)
  }
  
  lineTo(b, color = "black"){
    ctx.beginPath();
    ctx.moveTo(this.x, this.y);
    ctx.strokeStyle = color;
    ctx.lineTo(b.x, b.y);
    ctx.stroke();
  }
  
  lengthTo(b){
    return Math.sqrt((this.x - b.x) * (this.x - b.x) + (this.y - b.y)*(this.y - b.y));
  }
}

function clear(){
  ctx.fillStyle = "white";
  ctx.fillRect(0,0,ctx.canvas.width, ctx.canvas.height);
}

function percentLine(t, a, b){
  if (t == 1) return new Point(b.x, b.y);
  t = t / (1 - t);
  let x = (t*b.x + a.x) / (1 + t);
  let y = (t*b.y + a.y) / (1 + t);
  return new Point(x, y);
}

var t = 0;
var step = 0.005;

function bezierProc(step = 0.005){

  clear();
  var procPoints = [];
  for(let i = 0; i < points.length; i++){
    procPoints[i] = points[i];
  }
  let color_ind = 0;

  while(procPoints.length > 1){
    for(let i = 0; i < procPoints.length - 1; i++){
       procPoints[i].lineTo(procPoints[i+1], colors[color_ind]);
    }
    if(t != 1 && t != 0) color_ind =(color_ind + 1) % colors.length;
    
    for(let i = 0; i < procPoints.length - 1; i++){
      p = percentLine(t, procPoints[i], procPoints[i+1]);
      procPoints.splice(i, 1, p);
    }
    procPoints.splice(procPoints.length - 1, 1);
  }
  
  ctx.beginPath();
  ctx.arc(procPoints[0].x, procPoints[0].y, 4, 0, 2*Math.PI);
  ctx.fillStyle = "red";
  ctx.fill();
  
  drawBez4point(points);
  for(let point of points){
    point.draw();
  }
  
  if(t == 1){
    stop();
  }
  t+=step;
  if(t > 1){
    t = 1;
  }
  let print_t = Math.trunc(t * 1000) / 1000;
  document.getElementById("t_value").innerHTML = print_t;
}

function drawBez4point(points){
  ctx.beginPath();
  ctx.moveTo(points[0].x, points[0].y);
  for(let t = 0; t <= 1; t+= 0.005){
    let x = Math.pow((1-t),3)*points[0].x + 3*Math.pow((1-t),2)*t*points[1].x + 3*(1-t)*t*t*points[2].x + Math.pow(t,3)*points[3].x;
    let y = Math.pow((1-t),3)*points[0].y + 3*Math.pow((1-t),2)*t*points[1].y + 3*(1-t)*t*t*points[2].y + Math.pow(t,3)*points[3].y;
    ctx.lineTo(x,y);
  }  
  ctx.strokeStyle = "red";
  ctx.stroke();
}

function play(){
  if(!isRun){
    if(t == 1) init();
    isRun = true;
    Idint = setInterval(bezierProc, 30); 
  }
}
function stop(){
  if(isRun){
    isRun = false;
    clearInterval(Idint);
  }
}
function init(){
  t = 0;
  step = 0.005;
  bezierProc(0);
}
function resetAnim(){
  if(isRun){
    isRun = false;
    clearInterval(Idint);
  }
  init();
}

function main(){
  points.push(new Point(100, 150, "1"));
  points.push(new Point(250, 300, "2"));
  points.push(new Point(400, 50, "3"));
  points.push(new Point(550, 200, "4"));
  init();
}

main();